
from django.conf.urls import patterns, url

from crudbuilder.abstract import BaseBuilder
from crudbuilder.views import ViewBuilder
from crudbuilder import text

class UrlBuilder(BaseBuilder):
    def __init__(self, *args, **kwargs):
        result = super(UrlBuilder, self).__init__(*args, **kwargs)
        self.viewbuilder = ViewBuilder(*args, **kwargs)
        self.urls = self.generate_urls()

    def generate_urls(self):
        urls = []

        pluralized = text.plural(self.model)
        
        list_view = self.viewbuilder.generate_list_view()
        update_view = self.viewbuilder.generate_update_view()
        detail_view = self.viewbuilder.generate_detail_view()
        create_view = self.viewbuilder.generate_create_view()
        delete_view = self.viewbuilder.generate_delete_view()

        entries = [
            (r'^{}/{}/$', list_view.as_view(), '{}-{}-list'),
            (r'^{}/{}/(?P<pk>\d+)/$', detail_view.as_view(), '{}-{}-detail'),
            (r'^{}/{}/create/$', create_view.as_view(), '{}-{}-create'),
            (r'^{}/{}/(?P<pk>\d+)/update/$', update_view.as_view(), '{}-{}-update'),
            (r'^{}/{}/(?P<pk>\d+)/delete/$', delete_view.as_view(), '{}-{}-delete'),
        ]

        for entry in entries:
            address = entry[0].format(self.app, pluralized)
            url_name = entry[2].format(self.app, self.model)

            urls.append(
                url(address, entry[1], name=url_name),
            )
        return urls



from crudbuilder.text import mixedToUnder, plural

class CrudBuilderMixin(object):
	"""
	- Mixin to provide additional context data for the templates
	- app_label : return app_label for CRUD model
	- actual_model_name : returns the actual model name (person -> person)
	- pluralized_model_name : return the pluralized_model_name (person -> people)
	"""
	
	def get_context_data(self, **kwargs):
		context = super(CrudBuilderMixin, self).get_context_data(**kwargs)
		model = context['view'].model
		context['app_label'] = model._meta.app_label
		context['actual_model_name'] = mixedToUnder(model.__name__)
		context['pluralized_model_name'] = plural(model.__name__.lower())
		return context


from django.contrib.contenttypes.models import ContentType
import django_tables2 as tables
from django_tables2.utils import A

from crudbuilder.text import model_class_form, plural

class TableBuilder(object):
	def __init__(self, app, model, table_fields=None, css_table=None):
		self.app = app
		self.model = model
		self.table_fields = table_fields
		self.css_table = css_table

	def get_model_class(self):
		c = ContentType.objects.get(app_label=self.app, model=self.model)
		return c.model_class()

	def generate_table(self):
		model_class = self.get_model_class()
		detail_url_name = '{}-{}-detail'.format(self.app, self.model)

		main_attrs = dict(
			id = tables.LinkColumn(detail_url_name, args=[A('pk')])
			)
		meta_attrs = dict(
			model=model_class,
			fields=('id',) + self.table_fields if self.table_fields else ('id',),
			attrs={
				"class": "table table-bordered table-condensed",
				"empty_text" : "No {} exist".format(plural(self.model))
        	})

		main_attrs['Meta'] = type('Meta', (), meta_attrs)
		klass = type(model_class_form(self.model + 'Table'), (tables.Table,), main_attrs)
		return klass



import re
import string

__all__ = ['plural', 'mixedToUnder', 'capword', 'lowerword', 'underToMixed']


def plural(text):
    """
        >>> plural('activity')
        'activities'
    """
    aberrant = {
        'knife': 'knives',
        'self': 'selves',
        'elf': 'elves',
        'life': 'lives',
        'hoof': 'hooves',
        'leaf': 'leaves',
        'echo': 'echoes',
        'embargo': 'embargoes',
        'hero': 'heroes',
        'potato': 'potatoes',
        'tomato': 'tomatoes',
        'torpedo': 'torpedoes',
        'veto': 'vetoes',
        'child': 'children',
        'woman': 'women',
        'man': 'men',
        'person': 'people',
        'goose': 'geese',
        'mouse': 'mice',
        'barracks': 'barracks',
        'deer': 'deer',
        'nucleus': 'nuclei',
        'syllabus': 'syllabi',
        'focus': 'foci',
        'fungus': 'fungi',
        'cactus': 'cacti',
        'phenomenon': 'phenomena',
        'index': 'indices',
        'appendix': 'appendices',
        'criterion': 'criteria',


    }

    if text in aberrant:
        result = '%s' % aberrant[text]
    else:
        postfix = 's'
        if len(text) > 2:
            vowels = 'aeiou'
            if text[-2:] in ('ch', 'sh'):
                postfix = 'es'
            elif text[-1:] == 'y':
                if (text[-2:-1] in vowels) or (text[0] in string.uppercase):
                    postfix = 's'
                else:
                    postfix = 'ies'
                    text = text[:-1]
            elif text[-2:] == 'is':
                postfix = 'es'
                text = text[:-2]
            elif text[-1:] in ('s', 'z', 'x'):
                postfix = 'es'

        result = '%s%s' % (text, postfix)
    return result


class Style(object):

    """
    The base Style class, and also the simplest implementation.  No
    translation occurs -- column names and attribute names match,
    as do class names and table names (when using auto class or
    schema generation).
    """

    def __init__(self, pythonAttrToDBColumn=None,
                 dbColumnToPythonAttr=None,
                 pythonClassToDBTable=None,
                 dbTableToPythonClass=None,
                 idForTable=None,
                 longID=False):
        if pythonAttrToDBColumn:
            self.pythonAttrToDBColumn = lambda a, s = self: pythonAttrToDBColumn(
                s, a)
        if dbColumnToPythonAttr:
            self.dbColumnToPythonAttr = lambda a, s = self: dbColumnToPythonAttr(
                s, a)
        if pythonClassToDBTable:
            self.pythonClassToDBTable = lambda a, s = self: pythonClassToDBTable(
                s, a)
        if dbTableToPythonClass:
            self.dbTableToPythonClass = lambda a, s = self: dbTableToPythonClass(
                s, a)
        if idForTable:
            self.idForTable = lambda a, s = self: idForTable(s, a)
        self.longID = longID

    def pythonAttrToDBColumn(self, attr):
        return attr

    def dbColumnToPythonAttr(self, col):
        return col

    def pythonClassToDBTable(self, className):
        return className

    def dbTableToPythonClass(self, table):
        return table

    def idForTable(self, table):
        if self.longID:
            return self.tableReference(table)
        else:
            return 'id'

    def pythonClassToAttr(self, className):
        return lowerword(className)

    def instanceAttrToIDAttr(self, attr):
        return attr + "ID"

    def instanceIDAttrToAttr(self, attr):
        return attr[:-2]

    def tableReference(self, table):
        return table + "_id"


class MixedCaseUnderscoreStyle(Style):

    """
    This is the default style.  Python attributes use mixedCase,
    while database columns use underscore_separated.
    """

    def pythonAttrToDBColumn(self, attr):
        return mixedToUnder(attr)

    def dbColumnToPythonAttr(self, col):
        return underToMixed(col)

    def pythonClassToDBTable(self, className):
        return className[0].lower() \
            + mixedToUnder(className[1:])

    def dbTableToPythonClass(self, table):
        return table[0].upper() \
            + underToMixed(table[1:])

    def pythonClassToDBTableReference(self, className):
        return self.tableReference(self.pythonClassToDBTable(className))

    def tableReference(self, table):
        return table + "_id"

DefaultStyle = MixedCaseUnderscoreStyle


class MixedCaseStyle(Style):

    """
    This style leaves columns as mixed-case, and uses long
    ID names (like ProductID instead of simply id).
    """

    def pythonAttrToDBColumn(self, attr):
        return capword(attr)

    def dbColumnToPythonAttr(self, col):
        return lowerword(col)

    def dbTableToPythonClass(self, table):
        return capword(table)

    def tableReference(self, table):
        return table + "ID"

defaultStyle = DefaultStyle()


def getStyle(soClass, dbConnection=None):
    if dbConnection is None:
        if hasattr(soClass, '_connection'):
            dbConnection = soClass._connection
    if hasattr(soClass.sqlmeta, 'style') and soClass.sqlmeta.style:
        return soClass.sqlmeta.style
    elif dbConnection and dbConnection.style:
        return dbConnection.style
    else:
        return defaultStyle

#
# Text utilities
#
_mixedToUnderRE = re.compile(r'[A-Z]+')


def mixedToUnder(s):
    """
    Sample:
        >>> mixedToUnder("FooBarBaz")
        'foo_bar_baz'

    Special case for ID:
        >>> mixedToUnder("FooBarID")
        'foo_bar_id'
    """
    if s.endswith('ID'):
        return mixedToUnder(s[:-2] + "_id")
    trans = _mixedToUnderRE.sub(mixedToUnderSub, s)
    if trans.startswith('_'):
        trans = trans[1:]
    return trans


def mixedToUnderSub(match):
    m = match.group(0).lower()
    if len(m) > 1:
        return '_%s_%s' % (m[:-1], m[-1])
    else:
        return '_%s' % m


def capword(s):
    """
    >>> capword('foo')
    'Foo'
    """
    return s[0].upper() + s[1:]


def lowerword(s):
    """
    >>> lowerword('Hello')
    'hello'
    """
    return s[0].lower() + s[1:]

_underToMixedRE = re.compile('_.')


def underToMixed(name):
    """
    >>> underToMixed('some_large_model_name_perhaps')
    'someLargeModelNamePerhaps'

    >>> underToMixed('exception_for_id')
    'exceptionForID'
    """
    if name.endswith('_id'):
        return underToMixed(name[:-3] + "ID")
    return _underToMixedRE.sub(lambda m: m.group(0)[1].upper(),
                               name)


def model_class_form(name):
    """
    >>> model_class_form('foo_bar_baz')
    'FooBarBaz'
    """
    return capword(underToMixed(name))


def underToAllCaps(value):
    """
    >>> underToAllCaps('foo_bar_baz')
    'Foo Bar Baz'
    """
    return ' '.join(map(lambda x: x.title(), value.split('_')))


if __name__ == '__main__':
    print plural('activity')



import os

from django import forms
from django.core.urlresolvers import reverse_lazy
from django.contrib.contenttypes.models import ContentType
from django.forms.models import modelform_factory
from django.views.generic import(
    DetailView,
    ListView,
    CreateView,
    UpdateView,
    DeleteView
)
from django_tables2 import SingleTableView

from crudbuilder.mixins import CrudBuilderMixin
from crudbuilder.abstract import BaseBuilder
from crudbuilder.tables import TableBuilder
from crudbuilder.text import model_class_form, plural

class ViewBuilder(BaseBuilder):
    def __init__(self, *args, **kwargs):
        super(ViewBuilder, self).__init__(*args, **kwargs)
        self.classes = {}

    def generate_crud(self):
        self.generate_list_view()
        self.generate_create_view()
        self.generate_detail_view()
        self.generate_update_view()
        self.generate_delete_view()

    @property
    def get_model_class(self):
        c = ContentType.objects.get(app_label=self.app, model=self.model)
        return c.model_class()

    def get_actual_form(self):
        if self.custom_form:
            return self.custom_form
        else:
            return self.generate_modelform()

    def get_actual_table(self):
        if self.custom_table:
            return self.custom_table
        else:
            table_builder = TableBuilder(
                self.app,
                self.model,
                self.table_fields,
                self.css_table_class
                )

            return table_builder.generate_table()

    def generate_modelform(self):
        model_class = self.get_model_class
        excludes = self.form_excludes if self.form_excludes else []
        _ObjectForm = modelform_factory(model_class, exclude=excludes)
        return _ObjectForm

    def generate_list_view(self):
        name = model_class_form(self.model + 'ListView')

        list_args = dict(
            model=self.get_model_class,
            context_object_name=plural(self.model),
            template_name='object_list.html',
            table_class=self.get_actual_table(),
            context_table_name='table_objects',
            table_pagination=self.table_pagination
            )

        list_class = type(name, (CrudBuilderMixin, SingleTableView), list_args)
        self.classes[name] = list_class
        return list_class

    def generate_create_view(self):
        name = model_class_form(self.model + 'CreateView')
        create_args = dict(
            form_class=self.get_actual_form(),
            model=self.get_model_class,
            template_name='object_create.html',
            success_url=reverse_lazy('{}-{}-list'.format(self.app, self.model))
            )

        create_class = type(name, (CrudBuilderMixin, CreateView), create_args)
        self.classes[name] = create_class
        return create_class

    def generate_detail_view(self):
        name = model_class_form(self.model + 'DetailView')
        detail_args = dict(
            model=self.get_model_class,
            template_name='object_detail.html'
            )

        detail_class = type(name, (CrudBuilderMixin, DetailView), detail_args)
        self.classes[name] = detail_class
        return detail_class

    def generate_update_view(self):
        name = model_class_form(self.model + 'UpdateView')
        update_args = dict(
            form_class=self.get_actual_form(),
            model=self.get_model_class,
            template_name='object_update.html',
            success_url=reverse_lazy('{}-{}-list'.format(self.app, self.model))
            )

        update_class= type(name, (CrudBuilderMixin, UpdateView), update_args)
        self.classes[name] = update_class
        return update_class

    def generate_delete_view(self):
        name = model_class_form(self.model + 'DeleteView')
        delete_args = dict(
            model=self.get_model_class,
            template_name='object_delete.html',
            success_url=reverse_lazy('{}-{}-list'.format(self.app, self.model))
            )

        delete_class = type(name, (CrudBuilderMixin, DeleteView), delete_args)
        self.classes[name] = delete_class
        return delete_class




import os
from setuptools import setup
 
# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
 
setup(
    name = 'django-crudbuilder',
    version = '0.0.4',
    packages = ['crudbuilder'],
    include_package_data = True,
    license = 'BSD License',
    description = 'A simple Django CRUD builder',
    url = 'https://github.com/asifpy/django-crudbuilder/archive/master.tar.gz',
    author = 'Asif Jamadar',
    author_email = 'saluasif@gmail.com',
    install_requires=[
        'django_tables2>=1.0.4',
    ],
    classifiers =[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
    ]
)



class BaseBuilder(object):
	def __init__(self,
		app,
		model,
		form_excludes=None,
		custom_form=None,
		table_fields=None,
		css_table_class=None,
		custom_table=None,
		table_pagination=10):

		self.model = model
		self.app = app
		self.form_excludes = form_excludes
		self.custom_form = custom_form
		self.table_fields = table_fields
		self.css_table_class = css_table_class
		self.custom_table = custom_table
		self.table_pagination = table_pagination

